#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: 'zfb'
# time: 2019-07-12 10:28
import datetime
import time
import numpy as np

BASE_PATH = "/home/ps/xuekaiyu/AE-UNet3+/save_ionex/"
TEMPLATE_NAME = BASE_PATH + "template_header.txt"

# ionex文件的创建时间，例子：23-Apr-19 12:17
strfmt_create_time = "IRIS V2.0           IRIS                {}     PGM / RUN BY / DATE \n"
# now = time.localtime(time.time())
# create_time = strfmt_create_time.format(time.strftime('%d-%b-%y %H:%M',now))

# 当前属于该年份的第几天，例子：001, 2018
strfmt_day_index = "BUAA'S GLOBAL IONOSPHERE MAPS FOR DAY {}, {}             COMMENT             \n"
# day_time = strfmt_day_index.format(time.strftime('%j, %Y',now))

# TEC Map的开始日期，例子：2018  1  1
strfmt_tec_start_date = "  {}    {:2}    {:2}    {:2}     0     0                        EPOCH OF FIRST MAP  \n"

# TEC Map的结束日期，例子：2018  1  2
strfmt_tec_end_date = "  {}    {:2}    {:2}    {:2}     0     0                        EPOCH OF LAST MAP  \n"

# TEC Map的开始标志，例子：1
strfmt_tec_start_flag = "    {:2}                                                      START OF TEC MAP    \n"

# TEC Map的开始，例子：2018 1 1 0
strfmt_tec_start_time = "  {}    {:2}    {:2}    {:2}     0     0                        EPOCH OF CURRENT MAP\n"

# TEC Map的不同纬度的指示，例子：87.5
strfmt_tec_split_lat = "   {:5.1f}-180.0 180.0   5.0 450.0                            LAT/LON1/LON2/DLON/H\n"

# TEC Map的结束标志，例子：1
strfmt_tec_end_flag = "    {:2}                                                      END OF TEC MAP      \n"

# ionex文件的结束标志
strfmt_end_flag = "                                                            END OF FILE         "


def create_header(year, month, day):
    '''创建IONEX文件的头部

    根据年、月、日设置IONEX的header

    参数：
        year(int)：要创建的IONEX文件的年份
        month(int)：要创建的IONEX文件的月份
        day(int)：要创建的IONEX文件的日期
    
    返回值：
        content(list)：设置完毕的header，每个元素对应文件的一行
    '''
    with open(TEMPLATE_NAME, 'r+') as file:
        content = file.readlines()
        # 创建文件的时间
        now = time.localtime(time.time())
        str_create_time = strfmt_create_time.format(time.strftime('%d-%b-%y %H:%M',now))
        # 当前日期是这一年的第几天，例子：001
        day_index = datetime.date(year,month,day).strftime("%j")
        str_day_index = strfmt_day_index.format(day_index, year)
        # 文件中TEC Map的起始时刻
        str_tec_start_date = strfmt_tec_start_date.format(year, month, day, 0)
        # 文件中TEC Map的终止时刻
        str_tec_end_date = strfmt_tec_end_date.format(year, month, day, 23)
        # 开始替换模板头文件的对应行
        content[1] = str_create_time
        content[2] = str_day_index
        content[5] = str_tec_start_date
        content[6] = str_tec_end_date
        # 最后一行手动换行
        # content[-1] = content[-1]+"\n"
        return content

def create_body(year, month, day, tec_data):
    '''创建IONEX文件的主体

    根据年、月、日和TEC数据设置IONEX的主体

    参数：
        year(int)：要创建的IONEX文件的年份
        month(int)：要创建的IONEX文件的月份
        day(int)：要创建的IONEX文件的日期
        tec_data(np.array)：(24, 72, 80)的原始TEC矩阵

    返回值：
        body(list)：设置完毕的IONEX文件主体，每个元素对应文件的一行
    '''
    # tec_data包含frames个数据
    index_hour = 0
    body = []
    for hour_data in tec_data:
        # 每帧数据对应一个小时，shape=(72, 80)
        temp_rows, temp_cols = hour_data.shape
        # hour_data = np.delete(hour_data, temp_rows-1, axis=0)
        # hour_data = np.delete(hour_data, [x for x in range(temp_cols-7, temp_cols)], axis=1)
        # 此时的hour_data的shape=(71, 73)
        # 从87.5到-87.5共有71个纬度（间隔2.5度），从-180到180共有73个经度（间隔5度）
        str_tec_start_flag = strfmt_tec_start_flag.format(index_hour+1)
        str_tec_start_time = strfmt_tec_start_time.format(year, month, day, index_hour)
        body.append(str_tec_start_flag)
        body.append(str_tec_start_time)
        # 用来指示这是第几组纬度数据
        index_lat = 0
        for lat_data in hour_data:
            current_latitude = 87.5-2.5*index_lat
            str_tec_split_lat = strfmt_tec_split_lat.format(current_latitude)
            # 分割字符行
            body.append(str_tec_split_lat)
            # 真正的TEC Map数据，count用来控制每16个数据换行
            count = 0
            # 每16个数据组成一个字符串line
            line = ""
            # lat_data有73个浮点数据
            for data in lat_data:
                # 右对齐，占5个位置，只保留整数（四舍五入）
                line = line + format(data*10, ">5.0f")
                count = count + 1
                if count % 16 == 0:
                    body.append(line+"\n")
                    line = ""
            else:
                body.append(line+"\n")
            index_lat = index_lat+1
        # 当前时刻下的这组71,73的数据写入完毕，加上该时刻的结束标志
        str_tec_end_flag = strfmt_tec_end_flag.format(index_hour+1)
        body.append(str_tec_end_flag)
        index_hour = index_hour + 1
    body.append(strfmt_end_flag)
    return body


# name：生成文件的名字；year、month、day是tec_data对应的日期
def create_file(name, year, month, day, tec_data):
    '''创建IONEX文件

    根据名字、年、月、日和TEC数据创建IONEX格式文件

    参数：
        name(str)：要创建的IONEX文件名称
        year(int)：要创建的IONEX文件的年份
        month(int)：要创建的IONEX文件的月份
        day(int)：要创建的IONEX文件的日期
        tec_data(np.array)：(24, 72, 80)的原始TEC矩阵

    返回值：
        无
    '''
    with open(name, 'w+') as file:
        header = create_header(year, month, day)
        body = create_body(year, month, day, tec_data)
        # 合并连接两个list
        content = header + body
        file.writelines(content)


